#include <fstream>
#include <cassert>
#include "globalGameObject.h"
#include "userInput.h"
#include "timeOptionsStruct.h"
#include "prefPathWithAPI.h"
#include "actAccToInput.h"
#include "pieceCollisionTest.h"
#include "removeCompleteLines.h"
#include "moveCurrentPieceToMatrix.h"
#include "checkGameOver.h"
#include "bestScores.h"
#include "gameContext.h"

void runTetrisGame(std::ofstream& errorLog, const PrefPathFinder& prefPath, sf::RenderWindow& renderer, unsigned skillLevel, TimeOptions timeOpt, UserInput& input)
{
	GameObject gameObj{errorLog, skillLevel};
	bool quitGame{false};
	
	gameMainLoop(renderer, input, timeOpt, gameObj, quitGame);
	
	bestScores::bestScoresContext(errorLog, prefPath, renderer, input, gameObj.playScore, skillLevel);
	
	input.setEscapeKeyFalse();
	input.setWindowCloseFalse();
}

void gameMainLoop(sf::RenderWindow& renderer, UserInput& inp, TimeOptions& timeOpt, GameObject& gameObj, bool& quitGame)
{
	while( !quitGame )
	{
		inp.updateEvents(renderer);
		quitGameIfWindClose(inp, quitGame);
		actAccToPlayerInput(inp, gameObj);
		makeCurrentPieceGoDowner(gameObj, timeOpt, quitGame);
		modifyTimeData(timeOpt);
		removeCompleteLinesIfAny(gameObj.matrix, gameObj.playScore, gameObj.scanLinesInf);
		drawEverything(renderer, gameObj); 
	}
}

void makeCurrentPieceGoDowner(GameObject& gameObj, TimeOptions& timeOpt, bool& quitGame)
{
	if( gameObj.pieceFallTime.delaySufficient(timeOpt.fallDelay) )
	{
		if( !isPieceInCollisionWithMat(gameObj, 0, 1) && !hasPieceReachedBottom(gameObj ) )
		{
			gameObj.moveCurrentPieceDowner();
		}
		else{
			gameObj.movePieceToMatrix();
			gameObj.resetCurrentPiece();
			setGameOverIfMatrixFilled(gameObj.matrix, quitGame);
		}
	}
}

void drawEverything(sf::RenderWindow& renderer, GameObject& gameObj)
{
	if( gameObj.drawingFreq.delaySufficient(10) )
	{
		renderer.clear(sf::Color::Black);
		gameObj.drawEverything(renderer);
		renderer.display();
	}
}

void quitGameIfWindClose(UserInput& inp, bool& quitGame)
{
	if( inp.getWindowClose() || inp.getEscapeState() )
		quitGame = true;
}

void modifyTimeData(TimeOptions& timeOpt)
{
	if( timeOpt.changeFallTime.delaySufficient(PIECE_FALL_CHANGE_DELAY) )
	{
		if( timeOpt.fallDelay >= PIECE_FALL_TIME_MIN + PIECE_SPEED_INCR && timeOpt.fallDelay <= PIECE_FALL_TIME_MAX )
			timeOpt.fallDelay -= PIECE_SPEED_INCR;
	}
}