#include <string>
#include <array>
#include <experimental/filesystem>
#include "prefPathWithAPI.h"

namespace fs = std::experimental::filesystem;

#ifdef _WIN32

#include "windowsAPI.h"

PrefPathFinder::PrefPathFinder(const std::string& appDir):
	isAPathFound{false},
	errorMessage_{"No path was found"}
{
	TCHAR pathString[MAX_PATH];
	isAPathFound = winAPI_prefPath::composeNewPathIsSuccess(pathString, appDir, errorMessage_);
	
	if( isAPathFound )
	{
		prefPath_ = pathString;
		prefPath_ += "/";
	}
}

#endif //_WIN32

#ifdef __linux__

#include "unixAPI.h"

constexpr char UNIX_PREF_PATH[] = "/.local/share/";

PrefPathFinder::PrefPathFinder(const std::string& appDir):
	isAPathFound{false},
	errorMessage_{"No path was found"}
{
	std::string completePath;
	unixAPI::setHomeDir(completePath);
	completePath += UNIX_PREF_PATH;
	completePath += appDir;
	
	if( fs::exists(completePath) )
	{
		errorMessage_ = "The following directory: '" + completePath + "' already exists.\n";
		isAPathFound = true;
		prefPath_ = completePath;
		prefPath_ += "/";
	}
	else
	{
		errorMessage_ = "The following directory: '" + completePath + "' doesn't exist, creating it.\n";
		fs::create_directories(completePath);
		isAPathFound = true;
		prefPath_ = completePath;
		prefPath_ += "/";
	}
}

#endif //__linux__

const std::string& PrefPathFinder::getPath() const
{
	return prefPath_;
}

bool PrefPathFinder::isAValidPathFound() const
{
	return isAPathFound;
}

const std::string& PrefPathFinder::getError() const
{
	return errorMessage_;
}