#include <array>
#include "constexpr.h"
#include "tempMatrix.h"
#include "SFML/Window.hpp"
#include "SFML/Graphics.hpp"
#include "scanLinesInfos.h"
#include "playerScore.h"
#include "removeCompleteLines.h"

void removeCompleteLinesIfAny(MatrixTemp2D<SquareData>& matrix, PlayerScore& playerScore, ScanLinesInfos& scanLinesInf)
{
	if( scanLinesInf.canScanToRemoveLines )
	{
		size_t completeLinesNum{0};
		countCompleteLines(matrix, scanLinesInf, completeLinesNum);
		playerScore.addToScore( addCompletedLinesToScore(completeLinesNum) );
		playerScore.updateScoreText();
		decalOverCompleteLine(matrix, scanLinesInf);
		
		scanLinesInf.canScanToRemoveLines = false;
		scanLinesInf.bottomLine = 0;
	}
}

void countCompleteLines(const MatrixTemp2D<SquareData>& matrix, ScanLinesInfos& scanLinesInf, size_t& completeLinesNum)
{
	assert( scanLinesInf.bottomLine < matrix.height() );
	assert( scanLinesInf.topLine < matrix.height() );
	assert( scanLinesInf.bottomLine >= scanLinesInf.topLine );
	for( size_t height{scanLinesInf.topLine} ; height <= scanLinesInf.bottomLine ; ++height )
	{
		if( height >= 3 && height < matrix.height() )
		{
			if( isLineComplete(matrix, height) )
			{
				completeLinesNum++;
			}
		}
	}
}
 
bool isLineComplete(const MatrixTemp2D<SquareData>& matrix, size_t heightParam)
{
	assert( heightParam < matrix.height() );
	for( size_t width{0} ; width < matrix.width() ; ++width )
	{
		if( ! matrix(width, heightParam).filled  )
			return false;
	}
	return true;	
}

void decalOverCompleteLine(MatrixTemp2D<SquareData>& matrix, ScanLinesInfos& scanLinesInf)
{
	assert( scanLinesInf.bottomLine < matrix.height() );
	assert( scanLinesInf.topLine < matrix.height() );
	for( size_t height{scanLinesInf.bottomLine} ; height >= scanLinesInf.topLine ; --height )
	{
		if( height >= 3 && height < matrix.height() )
		{
			while( isLineComplete(matrix, height) )
			{
				browseTopLinesToMoveThem(matrix, height);
			}
		}
		else{
			return;
		}
	}
}

void browseTopLinesToMoveThem(MatrixTemp2D<SquareData>& matrix, size_t startBottom)
{
	for( size_t height{startBottom} ; height > 0 ; --height )
	{
		moveMatrixLineDowner(matrix, height);
	}
}

void moveMatrixLineDowner(MatrixTemp2D<SquareData>& matrix, size_t height)
{
	for( size_t width{0} ; width < matrix.width() ; ++width )
	{
		matrix(width, height) = matrix(width, height - 1);
		matrix(width, height - 1).filled = false;
	}
}

long unsigned addCompletedLinesToScore(unsigned completeLinesNum)
{
	switch( completeLinesNum )
	{
		case 1:
			return 10;
			break;
		case 2:
			return 40;
			break;
		case 3:
			return 120;
			break;
		case 4:
			return 320;
			break;
		default:
			return 0;
			break;
	}
}