#include <string>
#include <array>
#include <vector>
#include <fstream>
#include "constexpr.h"
#include "tempMatrix.h"
#include "loadedPieces.h"
#include "loadPieceFile.h"

void loadPieceFile(std::ofstream& errorLog, const std::string fileName, LoadedPieces& pieces)
{
	const std::string filePath{ "data/pieces/" + fileName };
	
	if( std::ifstream pieceFile{filePath} )
	{
		loadPieceFirstData(errorLog, filePath, pieceFile, pieces);
	}
	else{
		errorLog << "Error: couldn't open '" << filePath << "' piece file.\n";
	}
}

void loadPieceFirstData(std::ofstream& errorLog, const std::string& filePath, std::ifstream& inputFile, LoadedPieces& pieces)
{
	size_t matWidth{0}, matHeight{0};
	unsigned colorIndex{0};
		
	if( inputFile >> matWidth >> matHeight >> colorIndex )
	{
		MatrixTemp2D<SquareData> TemporaryMat{matWidth, matHeight};
		loadPieceMatrixData(errorLog, filePath, inputFile, TemporaryMat, pieces, colorIndex);
	}
	else{
		errorLog << "Error: in '" << filePath << "' file. Redaing mat width , height and color index error.\n";
	}
}

void loadPieceMatrixData(std::ofstream& errorLog, const std::string& fileName, std::ifstream& inputFile, MatrixTemp2D<SquareData>& matrix, LoadedPieces& pieces, unsigned color)
{
	bool data{0};
	for(size_t sqr_y{0} ; sqr_y < matrix.height() ; ++sqr_y )
	{
		for( size_t sqr_x{0} ; sqr_x < matrix.width() ; ++sqr_x )
		{
			if( inputFile >> data )
			{
				matrix(sqr_x, sqr_y).color = color;
				matrix(sqr_x, sqr_y).filled = data;
			}
			else{
				errorLog << "Error: couldn't read data x(" << sqr_x << ") y(" << sqr_y << ") for the following piece: " << fileName << '\n'; 
			}
		}
	}
	pieces.addPiece(matrix, color);
}

void loadPiecesAccToSkillLevel(std::ofstream& errorLog, unsigned skillLevel, LoadedPieces& pieces)
{
	switch( skillLevel )
	{
		case EASY_GAME:
			loadPieceFile(errorLog, "bigBar.piece", pieces);
			loadPieceFile(errorLog, "bigSquare.piece", pieces);
			loadPieceFile(errorLog, "littleBar.piece", pieces);
			loadPieceFile(errorLog, "littleSquare.piece", pieces);
			loadPieceFile(errorLog, "mediumBar.piece", pieces);
			break;
		case INTERMEDIATE_GAME:
			loadPieceFile(errorLog, "bigBar.piece", pieces);
			loadPieceFile(errorLog, "bigSquare.piece", pieces);
			loadPieceFile(errorLog, "corner.piece", pieces);
			loadPieceFile(errorLog, "knightMove.piece", pieces);
			loadPieceFile(errorLog, "littleBar.piece", pieces);
			loadPieceFile(errorLog, "littleSquare.piece", pieces);
			loadPieceFile(errorLog, "mediumBar.piece", pieces);
			break;
		case HARD_GAME:
			loadPieceFile(errorLog, "bigBar.piece", pieces);
			loadPieceFile(errorLog, "bigSquare.piece", pieces);
			loadPieceFile(errorLog, "bridge.piece", pieces);
			loadPieceFile(errorLog, "corner.piece", pieces);
			loadPieceFile(errorLog, "heap.piece", pieces);
			loadPieceFile(errorLog, "knightMove.piece", pieces);
			loadPieceFile(errorLog, "littleBar.piece", pieces);
			loadPieceFile(errorLog, "mediumBar.piece", pieces);
			loadPieceFile(errorLog, "zigzag.piece", pieces);
			break;
		default:
			assert( ! "You shouldn't have come here: wrong skill level" );
			break;
	}
}