#include <fstream>
#include <array>
#include "SFML/Window.hpp"
#include "SFML/Graphics.hpp"
#include "constexpr.h"
#include "userInput.h"
#include "timeDelayClass.h"
#include "timeOptionsStruct.h"
#include "highlightButton.h"
#include "skillLevelButtons.h"
#include "infosTextures.h"
#include "timeDelayClass.h"
#include "prefPathWithAPI.h"
#include "gameContext.h"
#include "mainMenu.h"

void mainMenu::context(std::ofstream& errorLog, const PrefPathFinder& prefPath, sf::RenderWindow& renderer)
{
	sf::Font Arial;
	if( Arial.loadFromFile(ARIALFONTPATH) )
	{
		bool quitMenu{false};
		skillChoice(errorLog, prefPath, renderer, Arial, quitMenu);
	}
	else{
		errorLog << "Error: couldn't open arial font.\n";
	}
}

void mainMenu::skillChoice(std::ofstream& errorLog, const PrefPathFinder& prefPath, sf::RenderWindow& renderer, sf::Font& arial, bool& quitMenu)
{
	SkillLevelButtons skillBtns{arial};
	InfosTextures keysInfos{arial};
	sf::Text title;
	initTitle(title, arial);
	UserInput inp{};
	TimeDelay delay{20};
	
	while( ! quitMenu )
	{
		inp.updateEvents(renderer);
		skillBtns.getUserEvents(inp.getMousePosX(), inp.getMousePosY(), inp.getMouseLeftButton());
		mainMenu::actWithButtons(errorLog, prefPath, renderer, skillBtns, inp);
		mainMenu::drawAll(renderer, title, skillBtns, keysInfos, delay);
		mainMenu::quitMenu(inp, quitMenu);
	}
}

void mainMenu::initTitle(sf::Text& title, sf::Font& arial)
{
	title.setFont(arial);
	title.setString("Tetris");
	title.setCharacterSize(24);
	title.setFillColor(sf::Color::White);
	title.setPosition(SCREENW/2 - title.getLocalBounds().width / 2 , SQR_SIZE *2);
}

void mainMenu::actWithButtons(std::ofstream& errorLog, const PrefPathFinder& prefPath, sf::RenderWindow& renderer, const SkillLevelButtons& skillBtns, UserInput& inp)
{
	for( size_t skill{0} ; skill < GAME_MAX ; ++skill )
	{
		if( skillBtns.isClicked(skill) )
		{
			runTetrisGame(errorLog, prefPath, renderer, skill, TimeOptions{ TimeDelay{}, PIECE_FALL_TIME_MAX }, inp);
			inp.setMouseLeftButtonFalse();
		}
	}
}

void mainMenu::drawAll(sf::RenderWindow& renderer, const sf::Text& title, const SkillLevelButtons& btns, const InfosTextures& keysInfos, TimeDelay& delay)
{
	if( delay.delaySufficient() )
	{
		renderer.clear(sf::Color::Black);
		renderer.draw(title);
		btns.drawButtons(renderer);
		keysInfos.drawAll(renderer);
		renderer.display();
	}
}

void mainMenu::quitMenu(UserInput& inp, bool& quit)
{
	if( inp.getEscapeState() || inp.getWindowClose() )
		quit = true;
}